#!/usr/bin/env node
global.navigator={};
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const degit_1 = __importDefault(require("degit"));
const fs_1 = require("fs");
const promises_1 = require("fs/promises");
const gitignore_parser_1 = __importDefault(require("gitignore-parser"));
const colors_1 = require("kleur/colors");
const path_1 = __importDefault(require("path"));
const prettier_1 = __importDefault(require("prettier"));
const parser_babel_1 = __importDefault(require("prettier/parser-babel"));
const prompts_1 = __importDefault(require("prompts"));
const tiny_glob_1 = __importDefault(require("tiny-glob"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const github_1 = require("./github");
const util_1 = require("./util");
const gitIgnore = `
dist
.uniform
.output
.vercel
.netlify
netlify
# dependencies
/node_modules
# IDEs and editors
/.idea
.project
.classpath
*.launch
.settings/
# Temp
gitignore
# System Files
.DS_Store
Thumbs.db
`;
const disclaimer = `
Welcome to the Uniform setup wizard!
There are definitely bugs and some features might not work yet.
If you encounter an issue, have a look at https://github.com/UniformFoundation/Uniform/issues and open a new one, if it is not already tracked.
`;
async function main() {
    const { version } = require('../package.json');
    console.log((0, colors_1.gray)(`\ncreate-uniform-app version ${version}`));
    console.log((0, colors_1.red)(disclaimer));
    const args = (0, yargs_parser_1.default)(process.argv.slice(2));
    const targetFolder = args[0] ||
        (await (0, prompts_1.default)({
            type: 'text',
            name: 'value',
            message: 'Where do you want to create',
            initial: 'my-app',
        })).value;
    if ((0, fs_1.existsSync)(targetFolder)) {
        if ((0, fs_1.readdirSync)(targetFolder).length > 0) {
            const response = await (0, prompts_1.default)({
                type: 'confirm',
                name: 'value',
                message: 'Directory not empty. Continue?',
                initial: false,
            });
            if (!response.value) {
                process.exit(1);
            }
        }
    }
    else {
        await (0, promises_1.mkdir)(targetFolder, {
            recursive: true,
        });
    }
    const config = {
        directory: args.example_dir ? args.example_dir : 'examples',
        repository: args.repo ? args.repo.split('/')[1] : 'Uniform',
        user: args.repo ? args.repo.split('/')[0] : 'UniformFoundation',
        ref: args.branch ? args.branch : 'main',
    };
    const templates = {};
    const templateDirs = (await (0, github_1.viaContentsApi)(config)).filter(d => d !== config.directory + '/' + '.DS_Store');
    templateDirs.forEach(dir => {
        let template = dir.replace('examples/', '');
        if (!templates[template]) {
            templates[template] = {
                name: template,
                js: true,
                ts: true,
            };
        }
    });
    const templateNames = [...Object.values(templates)];
    const templateName = (await (0, prompts_1.default)({
        type: 'select',
        name: 'template',
        message: 'Which template do you want to use?',
        choices: templateNames.map(template => ({ title: template.name, value: template.name })),
        initial: 0,
    })).template;
    if (!templateName) {
        throw new Error('No template selected');
    }
    const tempTemplate = path_1.default.join(targetFolder, '.uniform');
    await new Promise((res, rej) => {
        const emitter = (0, degit_1.default)(`${config.user}/${config.repository}/${config.directory}/${templateName}#${config.ref}`, {
            cache: false,
            force: true,
            verbose: true,
        });
        emitter.on('info', info => {
            const ignore = ['found matching commit hash'];
            if (!ignore.some(ignorant => info.message.startsWith(ignorant))) {
                console.log(info.message);
            }
        });
        emitter
            .clone(path_1.default.join(process.cwd(), tempTemplate))
            .then(() => {
            res({});
        })
            .catch(err => rej(err));
    });
    const templateDir = path_1.default.join(process.cwd(), tempTemplate);
    const gitignore_contents = gitIgnore;
    const gitignore = gitignore_parser_1.default.compile(gitignore_contents);
    const files = (await (0, tiny_glob_1.default)('**/*', { cwd: templateDir })).filter(gitignore.accepts);
    // let writtenFiles = 1;
    await Promise.all(files.map(async (file) => {
        // console.log(green(`${writtenFiles} / ${files.length} written...`));
        const src = path_1.default.join(templateDir, file);
        const dest = path_1.default.join(targetFolder, file);
        if ((0, fs_1.statSync)(src).isDirectory()) {
            await (0, promises_1.mkdir)(dest, {
                recursive: true,
            });
        }
        else {
            let code = (await (0, promises_1.readFile)(src)).toString();
            // if (src.includes('vite.config') && !code.includes('ssr: false') && !ssr) {
            //     code = code.replace(`solid({`, `solid({ ssr: false, `).replace(`solid()`, `solid({ ssr: false })`);
            // }
            if (src.endsWith('.ts')) {
                await (0, promises_1.writeFile)(dest, prettier_1.default.format(code, { parser: 'babel-ts', plugins: [parser_babel_1.default] }));
            }
            else {
                await (0, promises_1.copyFile)(src, dest);
            }
        }
    }));
    await (0, promises_1.writeFile)(path_1.default.join(targetFolder, '.gitignore'), gitignore_contents);
    const name = path_1.default.basename(path_1.default.resolve(targetFolder));
    const pkg_file = path_1.default.join(targetFolder, 'package.json');
    const pkg_json = JSON.parse((0, fs_1.readFileSync)(pkg_file, 'utf-8')
        .replace(/"name": ".+"/, _m => `"name": "${name}"`)
        .replace(/"(.+)": "workspace:.+"/g, (_m, name) => `"${name}": "next"`)); // TODO ^${versions[name]}
    await (0, promises_1.writeFile)(pkg_file, JSON.stringify(pkg_json, null, 2));
    await (0, promises_1.rm)(path_1.default.join(process.cwd(), tempTemplate), {
        recursive: true,
        force: true,
    });
    console.log((0, colors_1.bold)((0, colors_1.green)('✔ Copied project files')));
    console.log('\nNext steps:');
    let i = 1;
    const relative = path_1.default.relative(process.cwd(), targetFolder);
    if (relative !== '') {
        console.log(`  ${i++}: ${(0, colors_1.bold)((0, colors_1.cyan)(`cd ${relative}`))}`);
    }
    const userPkgManager = (0, util_1.getUserPkgManager)();
    console.log(`  ${i++}: ${(0, colors_1.bold)((0, colors_1.cyan)(`${userPkgManager} install`))}`);
    const devCommand = [`${userPkgManager} run dev`, userPkgManager === 'npm' ? '--' : ''].filter(Boolean).join(' ');
    console.log(`  ${i++}: ${(0, colors_1.bold)((0, colors_1.cyan)(devCommand))}`);
    console.log(`\nTo close the dev server, hit ${(0, colors_1.bold)((0, colors_1.cyan)('Ctrl-C'))}`);
}
exports.default = main;
main().catch(console.error);
